/*
 ThermostatQA - test framework for Thermostat Monitoring Tool

 Copyright 2015 Red Hat, Inc.

 This file is part of ThermostatQA

 ThermostatQA is distributed under the GNU General Public License,
 version 2 or any later version (with a special exception described
 below, commonly known as the "Classpath Exception").

 A copy of GNU General Public License (GPL) is included in this
 distribution, in the file COPYING.

 Linking ThermostatQA code with other modules is making a combined work
 based on ThermostatQA.  Thus, the terms and conditions of the GPL
 cover the whole combination.

 As a special exception, the copyright holders of ThermostatQA give you
 permission to link this code with independent modules to produce an
 executable, regardless of the license terms of these independent
 modules, and to copy and distribute the resulting executable under
 terms of your choice, provided that you also meet, for each linked
 independent module, the terms and conditions of the license of that
 module.  An independent module is a module which is not derived from
 or based on ThermostatQA code.  If you modify ThermostatQA, you may
 extend this exception to your version of the software, but you are
 not obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version.
 */
package org.thermostat.qa2.tests;

import static org.thermostat.qa2.framework.Assert.assertRuns;
import org.thermostat.qa2.framework.Robot;
import org.thermostat.qa2.framework.ThermostatQAConfig;
import org.thermostat.qa2.framework.annotations.SetupStorage;
import org.thermostat.qa2.framework.annotations.RunGnomeKeyring;
import org.thermostat.qa2.framework.annotations.RunStorage;
import org.thermostat.qa2.framework.annotations.Test;
import org.thermostat.qa2.framework.services.ThermostatGui;
import org.thermostat.qa2.framework.services.ThermostatStorage;
import org.thermostat.qa2.framework.services.Tomcat;

/**
 * Test class ported from old framework
 * @author Zdeněk Žamberský
 */
@SetupStorage(type = "web")
@RunGnomeKeyring
public class GuiClientWebStorageTest {

    static final String[] thermGuiWithTomcatOnlyThermostatLogs = AgentWebStorageTest.THERM_AGENT_WITH_TOMCAT_ONLY_THERMOSTAT_LOGS;
    static final String[] thermGuiWithTomcatOnlyCatalinaLogs = AgentWebStorageTest.THERM_AGENT_WITH_TOMCAT_ONLY_CATALINA_LOGS;
    static final String[] thermGuiWithWebStorageThermostatLogs = AgentWebStorageTest.THERM_AGENT_WITH_WEB_STORAGE_THERMOSTAT_LOGS;
    static final String[] thermGuiWithWebStorageCatalinaLogs = AgentWebStorageTest.THERM_AGENT_WITH_WEB_STORAGE_CATALINA_LOGS;
    static final String[] thermGuiBadLoginCatalinaLogs = {
        "org.apache.catalina.realm.JAASRealm authenticate",
        "WARNING: Login exception authenticating username \"time-for-tea\"",
        "javax.security.auth.login.LoginException: User 'time-for-tea' not found"
    };

    public static void checkForFailedConnectionPatterns(Robot robot) throws Exception {
        robot.checkForPattern("FailureToConnectToStorageDialog/error_icon");
        robot.checkForPattern("FailureToConnectToStorageDialog/configure_button");
        robot.checkForPattern("FailureToConnectToStorageDialog/quit_button");
    }

    public static void checkForMainWindowPatterns(Robot robot) throws Exception {
        robot.checkForPattern("MainMenu/menu_bar_file");
        robot.checkForPattern("MainMenu/menu_bar_edit");
        robot.checkForPattern("MainMenu/menu_bar_view");
        robot.checkForPattern("MainMenu/menu_bar_help");
    }

    @Test
    public void startWithTomcatWithoutStorage() throws Exception {
        Tomcat tomcat = new Tomcat();
        tomcat.start();

        ThermostatGui gui = new ThermostatGui();
        gui.start();
        assertRuns(gui, "Gui should be running");

        Robot robot = new Robot();
        robot.makeScreenshot("FailureToConnectToWebStorageDialog01");
        checkForFailedConnectionPatterns(robot);
        robot.saveScreenshot("FailureToConnectToWebStorageDialog02");
    }

    @Test
    public void startWithStorageWithoutTomcat() throws Exception {
        ThermostatStorage storage = new ThermostatStorage();
        storage.start();

        ThermostatGui gui = new ThermostatGui();
        gui.start();
        assertRuns(gui, "Gui should be running");

        Robot robot = new Robot();
        robot.makeScreenshot("FailureToConnectToWebStorageDialog11");
        checkForFailedConnectionPatterns(robot);
        robot.saveScreenshot("FailureToConnectToWebStorageDialog12");
    }

    @Test
    public void startGuiAndCheckTomcatLogs() throws Exception {
        ThermostatStorage storage = new ThermostatStorage();
        storage.start();

        Tomcat tomcat = new Tomcat();
        tomcat.start();

        ThermostatGui gui = new ThermostatGui();
        gui.start();
        assertRuns(gui, "Gui should be running");

        Robot robot = new Robot();
        robot.makeScreenshot("GuiWithWebStorage1");
        checkForMainWindowPatterns(robot);
        robot.saveScreenshot("GuiWithWebStorage2");

        gui.stop();
        tomcat.stop();
        storage.stop();

        AgentWebStorageTest.checkForPatterns("catalina", thermGuiWithWebStorageCatalinaLogs);
        AgentWebStorageTest.checkForPatterns("thermostat-web-storage.", thermGuiWithWebStorageThermostatLogs);
    }

    @Test
    @SetupStorage(type = "web", badClientLogin = true)
    public void startGuiWithBadLogin() throws Exception {
        ThermostatStorage storage = new ThermostatStorage();
        storage.start();

        Tomcat tomcat = new Tomcat();
        tomcat.start();

        ThermostatGui gui = new ThermostatGui();
        gui.start();
        assertRuns(gui, "Gui should be running");

        Robot robot = new Robot();
        robot.makeScreenshot("guiWSBadLogin");
        checkForFailedConnectionPatterns(robot);
        robot.saveScreenshot("guiWSBadLogin2");

        gui.stop();
        tomcat.stop();
        storage.stop();

        //AgentWebStorageTest.checkForPatterns("catalina", thermGuiBadLoginCatalinaLogs);
    }

    @Test
    @SetupStorage(type = "web", badClientLogin = true)
    public void guiFixBadLoginWebStorage() throws Exception {
        ThermostatStorage storage = new ThermostatStorage();
        storage.start();

        Tomcat tomcat = new Tomcat();
        tomcat.start();

        ThermostatGui gui = new ThermostatGui();
        gui.start();
        assertRuns(gui, "Gui should be running");

        Robot robot = new Robot();
        robot.makeScreenshot("guiWSBadLogin");
        robot.clickOnPattern("FailureToConnectToStorageDialog/configure_button");

        robot.makeScreenshot("guiWSConfigure");
        robot.checkForPattern("ClientPreferencesDialog/web_storage_connection_info");

        ThermostatQAConfig.Login login = ThermostatQAConfig.clientLogin;
        robot.doubleClickOnPattern("ClientPreferencesDialog/bad_login_user_name");
        robot.writeText(login.getUsername());

        robot.doubleClickOnPattern("ClientPreferencesDialog/password_edit_and_label");
        robot.writeText(login.getPassword());

        robot.clickOnPattern("ClientPreferencesDialog/ok_button");
        assertRuns(gui, "Gui should be running");

        robot.makeScreenshot("guiWSFixedLogin");
        checkForMainWindowPatterns(robot);

        gui.stop();
        tomcat.stop();
        storage.stop();

        //AgentWebStorageTest.checkForPatterns("catalina", thermGuiBadLoginCatalinaLogs);
        AgentWebStorageTest.checkForPatterns("catalina", thermGuiWithWebStorageCatalinaLogs);
        AgentWebStorageTest.checkForPatterns("thermostat-web-storage.", thermGuiWithWebStorageThermostatLogs);
    }

    @Test
    public void startGUIWithoutWebStorage() throws Exception {
        ThermostatGui gui = new ThermostatGui();
        gui.start();
        assertRuns(gui, "Gui should be running");

        Robot robot = new Robot();
        robot.makeScreenshot("FailureToConnectToWebStorageDialog1");
        checkForFailedConnectionPatterns(robot);
        robot.saveScreenshot("FailureToConnectToWebStorageDialog2");
    }

    @Test
    @RunStorage
    public void startGUIWithWebStorageMenuTest() throws Exception {
        ThermostatGui gui = new ThermostatGui();
        gui.start();
        assertRuns(gui, "Gui should be running");

        Robot robot = new Robot();
        robot.makeScreenshot("testStartGUIWithWebStorageMenuTest1");
        checkForMainWindowPatterns(robot);
        robot.saveScreenshot("testStartGUIWithWebStorageMenuTest2");
    }

    @Test
    @RunStorage
    public void editClientPreferences() throws Exception {
        ThermostatGui gui = new ThermostatGui();
        gui.start();
        assertRuns(gui, "Gui should be running");

        Robot robot = new Robot();
        robot.enterMainMenu();
        robot.pressRightKey();
        robot.pressDownKey();
        robot.pressEnterKey();
        assertRuns(gui, "Gui should be running");

        robot.makeScreenshot("testEditClientPreferencesWSTest1");
        robot.checkForPattern("ClientPreferencesDialog/ok_button");
        robot.checkForPattern("ClientPreferencesDialog/cancel_button");
        robot.checkForPattern("ClientPreferencesDialog/connection_info_label");
        robot.checkForPattern("ClientPreferencesDialog/user_name_label");
        robot.checkForPattern("ClientPreferencesDialog/password_label");
        robot.checkForPattern("ClientPreferencesDialog/storage_url_label");
        robot.checkForPattern("ClientPreferencesDialog/save_entitlements_checked");
        robot.saveScreenshot("testEditClientPreferencesWSTest2");
    }
}
